package java.time;

import java.io.ObjectInput;
import java.io.IOException;
import java.io.ObjectOutput;
import java.io.InvalidObjectException;
import java.io.ObjectInputStream;
import java.time.chrono.IsoChronology;
import java.time.temporal.TemporalQuery;
import java.time.temporal.TemporalAmount;
import java.time.temporal.UnsupportedTemporalTypeException;
import java.time.temporal.ValueRange;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalUnit;
import java.time.temporal.ChronoField;
import java.time.temporal.TemporalField;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalQueries;
import java.time.temporal.TemporalAccessor;
import java.util.Objects;
import java.util.Comparator;
import java.io.Serializable;
import java.time.temporal.TemporalAdjuster;
import java.time.temporal.Temporal;

public final class OffsetDateTime implements Temporal, TemporalAdjuster, Comparable<OffsetDateTime>, Serializable
{
    public static final OffsetDateTime MIN;
    public static final OffsetDateTime MAX;
    private static final long serialVersionUID = 2287754244819255394L;
    private final LocalDateTime dateTime;
    private final ZoneOffset offset;
    
    public static Comparator<OffsetDateTime> timeLineOrder() {
        return OffsetDateTime::compareInstant;
    }
    
    private static int compareInstant(OffsetDateTime offsetDateTime, OffsetDateTime offsetDateTime2) {
        if (offsetDateTime.getOffset().equals(offsetDateTime2.getOffset())) {
            return offsetDateTime.toLocalDateTime().compareTo(offsetDateTime2.toLocalDateTime());
        }
        int compare = Long.compare(offsetDateTime.toEpochSecond(), offsetDateTime2.toEpochSecond());
        if (compare == 0) {
            compare = offsetDateTime.toLocalTime().getNano() - offsetDateTime2.toLocalTime().getNano();
        }
        return compare;
    }
    
    public static OffsetDateTime now() {
        return now(Clock.systemDefaultZone());
    }
    
    public static OffsetDateTime now(ZoneId zoneId) {
        return now(Clock.system(zoneId));
    }
    
    public static OffsetDateTime now(Clock obj) {
        Objects.requireNonNull(obj, "clock");
        Instant instant = obj.instant();
        return ofInstant(instant, obj.getZone().getRules().getOffset(instant));
    }
    
    public static OffsetDateTime of(LocalDate localDate, LocalTime localTime, ZoneOffset zoneOffset) {
        return new OffsetDateTime(LocalDateTime.of(localDate, localTime), zoneOffset);
    }
    
    public static OffsetDateTime of(LocalDateTime localDateTime, ZoneOffset zoneOffset) {
        return new OffsetDateTime(localDateTime, zoneOffset);
    }
    
    public static OffsetDateTime of(int n, int n2, int n3, int n4, int n5, int n6, int n7, ZoneOffset zoneOffset) {
        return new OffsetDateTime(LocalDateTime.of(n, n2, n3, n4, n5, n6, n7), zoneOffset);
    }
    
    public static OffsetDateTime ofInstant(Instant instant, ZoneId obj) {
        Objects.requireNonNull(instant, "instant");
        Objects.requireNonNull(obj, "zone");
        ZoneOffset offset = obj.getRules().getOffset(instant);
        return new OffsetDateTime(LocalDateTime.ofEpochSecond(instant.getEpochSecond(), instant.getNano(), offset), offset);
    }
    
    public static OffsetDateTime from(TemporalAccessor obj) {
        if ((obj instanceof OffsetDateTime)) {
            return (OffsetDateTime)obj;
        }
        try {
            ZoneOffset from = ZoneOffset.from(obj);
            LocalDate localDate = (LocalDate)obj.query(TemporalQueries.localDate());
            LocalTime localTime = (LocalTime)obj.query(TemporalQueries.localTime());
            if (localDate != null && localTime != null) {
                return of(localDate, localTime, from);
            }
            return ofInstant(Instant.from(obj), from);
        }
        catch (DateTimeException ex) {
            throw new DateTimeException("Unable to obtain OffsetDateTime from TemporalAccessor: " + obj + " of type " + obj.getClass().getName(), ex);
        }
    }
    
    public static OffsetDateTime parse(CharSequence charSequence) {
        return parse(charSequence, DateTimeFormatter.ISO_OFFSET_DATE_TIME);
    }
    
    public static OffsetDateTime parse(CharSequence text, DateTimeFormatter obj) {
        Objects.requireNonNull(obj, "formatter");
        return (OffsetDateTime)obj.parse(text, OffsetDateTime::from);
    }
    
    private OffsetDateTime(LocalDateTime obj, ZoneOffset obj2) {
        this.dateTime = (LocalDateTime)Objects.requireNonNull(obj, "dateTime");
        this.offset = (ZoneOffset)Objects.requireNonNull(obj2, "offset");
    }
    
    private OffsetDateTime with(LocalDateTime localDateTime, ZoneOffset zoneOffset) {
        if (this.dateTime == localDateTime && this.offset.equals(zoneOffset)) {
            return this;
        }
        return new OffsetDateTime(localDateTime, zoneOffset);
    }
    
    @Override
    public boolean isSupported(TemporalField temporalField) {
        return temporalField instanceof ChronoField || (temporalField != null && temporalField.isSupportedBy(this));
    }
    
    @Override
    public boolean isSupported(TemporalUnit temporalUnit) {
        if ((temporalUnit instanceof ChronoUnit)) {
            return temporalUnit != ChronoUnit.FOREVER;
        }
        return temporalUnit != null && temporalUnit.isSupportedBy(this);
    }
    
    @Override
    public ValueRange range(TemporalField temporalField) {
        if ((temporalField instanceof ChronoField)) {
            if (temporalField == ChronoField.INSTANT_SECONDS || temporalField == ChronoField.OFFSET_SECONDS) {
                return temporalField.range();
            }
            return this.dateTime.range(temporalField);
        }
        else {
            return temporalField.rangeRefinedBy(this);
        }
    }
    
    @Override
    public int get(TemporalField field) {
        if ((field instanceof ChronoField)) {
            switch ((ChronoField)field) {
                case INSTANT_SECONDS: {
                    throw new UnsupportedTemporalTypeException("Invalid field 'InstantSeconds' for get() method, use getLong() instead");
                }
                case OFFSET_SECONDS: {
                    return this.getOffset().getTotalSeconds();
                }
                default: {
                    return this.dateTime.get(field);
                }
            }
        }
        else {
            return super.get(field);
        }
    }
    
    @Override
    public long getLong(TemporalField temporalField) {
        if ((temporalField instanceof ChronoField)) {
            switch ((ChronoField)temporalField) {
                case INSTANT_SECONDS: {
                    return this.toEpochSecond();
                }
                case OFFSET_SECONDS: {
                    return (long)this.getOffset().getTotalSeconds();
                }
                default: {
                    return this.dateTime.getLong(temporalField);
                }
            }
        }
        else {
            return temporalField.getFrom(this);
        }
    }
    
    public ZoneOffset getOffset() {
        return this.offset;
    }
    
    public OffsetDateTime withOffsetSameLocal(ZoneOffset zoneOffset) {
        return this.with(this.dateTime, zoneOffset);
    }
    
    public OffsetDateTime withOffsetSameInstant(ZoneOffset zoneOffset) {
        if (zoneOffset.equals(this.offset)) {
            return this;
        }
        return new OffsetDateTime(this.dateTime.plusSeconds((long)(zoneOffset.getTotalSeconds() - this.offset.getTotalSeconds())), zoneOffset);
    }
    
    public LocalDateTime toLocalDateTime() {
        return this.dateTime;
    }
    
    public LocalDate toLocalDate() {
        return this.dateTime.toLocalDate();
    }
    
    public int getYear() {
        return this.dateTime.getYear();
    }
    
    public int getMonthValue() {
        return this.dateTime.getMonthValue();
    }
    
    public Month getMonth() {
        return this.dateTime.getMonth();
    }
    
    public int getDayOfMonth() {
        return this.dateTime.getDayOfMonth();
    }
    
    public int getDayOfYear() {
        return this.dateTime.getDayOfYear();
    }
    
    public DayOfWeek getDayOfWeek() {
        return this.dateTime.getDayOfWeek();
    }
    
    public LocalTime toLocalTime() {
        return this.dateTime.toLocalTime();
    }
    
    public int getHour() {
        return this.dateTime.getHour();
    }
    
    public int getMinute() {
        return this.dateTime.getMinute();
    }
    
    public int getSecond() {
        return this.dateTime.getSecond();
    }
    
    public int getNano() {
        return this.dateTime.getNano();
    }
    
    @Override
    public OffsetDateTime with(TemporalAdjuster temporalAdjuster) {
        if (temporalAdjuster instanceof LocalDate || temporalAdjuster instanceof LocalTime || temporalAdjuster instanceof LocalDateTime) {
            return this.with(this.dateTime.with(temporalAdjuster), this.offset);
        }
        if ((temporalAdjuster instanceof Instant)) {
            return ofInstant((Instant)temporalAdjuster, this.offset);
        }
        if ((temporalAdjuster instanceof ZoneOffset)) {
            return this.with(this.dateTime, (ZoneOffset)temporalAdjuster);
        }
        if ((temporalAdjuster instanceof OffsetDateTime)) {
            return (OffsetDateTime)temporalAdjuster;
        }
        return (OffsetDateTime)temporalAdjuster.adjustInto(this);
    }
    
    @Override
    public OffsetDateTime with(TemporalField temporalField, long value) {
        if ((temporalField instanceof ChronoField)) {
            ChronoField chronoField = (ChronoField)temporalField;
            switch (chronoField) {
                case INSTANT_SECONDS: {
                    return ofInstant(Instant.ofEpochSecond(value, (long)this.getNano()), this.offset);
                }
                case OFFSET_SECONDS: {
                    return this.with(this.dateTime, ZoneOffset.ofTotalSeconds(chronoField.checkValidIntValue(value)));
                }
                default: {
                    return this.with(this.dateTime.with(temporalField, value), this.offset);
                }
            }
        }
        else {
            return (OffsetDateTime)temporalField.adjustInto(this, value);
        }
    }
    
    public OffsetDateTime withYear(int n) {
        return this.with(this.dateTime.withYear(n), this.offset);
    }
    
    public OffsetDateTime withMonth(int n) {
        return this.with(this.dateTime.withMonth(n), this.offset);
    }
    
    public OffsetDateTime withDayOfMonth(int n) {
        return this.with(this.dateTime.withDayOfMonth(n), this.offset);
    }
    
    public OffsetDateTime withDayOfYear(int n) {
        return this.with(this.dateTime.withDayOfYear(n), this.offset);
    }
    
    public OffsetDateTime withHour(int n) {
        return this.with(this.dateTime.withHour(n), this.offset);
    }
    
    public OffsetDateTime withMinute(int n) {
        return this.with(this.dateTime.withMinute(n), this.offset);
    }
    
    public OffsetDateTime withSecond(int n) {
        return this.with(this.dateTime.withSecond(n), this.offset);
    }
    
    public OffsetDateTime withNano(int n) {
        return this.with(this.dateTime.withNano(n), this.offset);
    }
    
    public OffsetDateTime truncatedTo(TemporalUnit temporalUnit) {
        return this.with(this.dateTime.truncatedTo(temporalUnit), this.offset);
    }
    
    @Override
    public OffsetDateTime plus(TemporalAmount temporalAmount) {
        return (OffsetDateTime)temporalAmount.addTo(this);
    }
    
    @Override
    public OffsetDateTime plus(long n, TemporalUnit temporalUnit) {
        if ((temporalUnit instanceof ChronoUnit)) {
            return this.with(this.dateTime.plus(n, temporalUnit), this.offset);
        }
        return (OffsetDateTime)temporalUnit.addTo(this, n);
    }
    
    public OffsetDateTime plusYears(long n) {
        return this.with(this.dateTime.plusYears(n), this.offset);
    }
    
    public OffsetDateTime plusMonths(long n) {
        return this.with(this.dateTime.plusMonths(n), this.offset);
    }
    
    public OffsetDateTime plusWeeks(long n) {
        return this.with(this.dateTime.plusWeeks(n), this.offset);
    }
    
    public OffsetDateTime plusDays(long n) {
        return this.with(this.dateTime.plusDays(n), this.offset);
    }
    
    public OffsetDateTime plusHours(long n) {
        return this.with(this.dateTime.plusHours(n), this.offset);
    }
    
    public OffsetDateTime plusMinutes(long n) {
        return this.with(this.dateTime.plusMinutes(n), this.offset);
    }
    
    public OffsetDateTime plusSeconds(long n) {
        return this.with(this.dateTime.plusSeconds(n), this.offset);
    }
    
    public OffsetDateTime plusNanos(long n) {
        return this.with(this.dateTime.plusNanos(n), this.offset);
    }
    
    @Override
    public OffsetDateTime minus(TemporalAmount temporalAmount) {
        return (OffsetDateTime)temporalAmount.subtractFrom(this);
    }
    
    @Override
    public OffsetDateTime minus(long n, TemporalUnit temporalUnit) {
        return (n == Long.MIN_VALUE) ? this.plus(Long.MAX_VALUE, temporalUnit).plus(1L, temporalUnit) : this.plus(-n, temporalUnit);
    }
    
    public OffsetDateTime minusYears(long n) {
        return (n == Long.MIN_VALUE) ? this.plusYears(Long.MAX_VALUE).plusYears(1L) : this.plusYears(-n);
    }
    
    public OffsetDateTime minusMonths(long n) {
        return (n == Long.MIN_VALUE) ? this.plusMonths(Long.MAX_VALUE).plusMonths(1L) : this.plusMonths(-n);
    }
    
    public OffsetDateTime minusWeeks(long n) {
        return (n == Long.MIN_VALUE) ? this.plusWeeks(Long.MAX_VALUE).plusWeeks(1L) : this.plusWeeks(-n);
    }
    
    public OffsetDateTime minusDays(long n) {
        return (n == Long.MIN_VALUE) ? this.plusDays(Long.MAX_VALUE).plusDays(1L) : this.plusDays(-n);
    }
    
    public OffsetDateTime minusHours(long n) {
        return (n == Long.MIN_VALUE) ? this.plusHours(Long.MAX_VALUE).plusHours(1L) : this.plusHours(-n);
    }
    
    public OffsetDateTime minusMinutes(long n) {
        return (n == Long.MIN_VALUE) ? this.plusMinutes(Long.MAX_VALUE).plusMinutes(1L) : this.plusMinutes(-n);
    }
    
    public OffsetDateTime minusSeconds(long n) {
        return (n == Long.MIN_VALUE) ? this.plusSeconds(Long.MAX_VALUE).plusSeconds(1L) : this.plusSeconds(-n);
    }
    
    public OffsetDateTime minusNanos(long n) {
        return (n == Long.MIN_VALUE) ? this.plusNanos(Long.MAX_VALUE).plusNanos(1L) : this.plusNanos(-n);
    }
    
    @Override
    public <R> R query(TemporalQuery<R> temporalQuery) {
        if (temporalQuery == TemporalQueries.offset() || temporalQuery == TemporalQueries.zone()) {
            return this.getOffset();
        }
        if (temporalQuery == TemporalQueries.zoneId()) {
            return null;
        }
        if (temporalQuery == TemporalQueries.localDate()) {
            return this.toLocalDate();
        }
        if (temporalQuery == TemporalQueries.localTime()) {
            return this.toLocalTime();
        }
        if (temporalQuery == TemporalQueries.chronology()) {
            return IsoChronology.INSTANCE;
        }
        if (temporalQuery == TemporalQueries.precision()) {
            return ChronoUnit.NANOS;
        }
        return temporalQuery.queryFrom(this);
    }
    
    @Override
    public Temporal adjustInto(Temporal temporal) {
        return temporal.with(ChronoField.EPOCH_DAY, this.toLocalDate().toEpochDay()).with(ChronoField.NANO_OF_DAY, this.toLocalTime().toNanoOfDay()).with(ChronoField.OFFSET_SECONDS, (long)this.getOffset().getTotalSeconds());
    }
    
    @Override
    public long until(Temporal temporal, TemporalUnit temporalUnit) {
        OffsetDateTime from = from(temporal);
        if ((temporalUnit instanceof ChronoUnit)) {
            return this.dateTime.until(from.withOffsetSameInstant(this.offset).dateTime, temporalUnit);
        }
        return temporalUnit.between(this, from);
    }
    
    public String format(DateTimeFormatter obj) {
        Objects.requireNonNull(obj, "formatter");
        return obj.format(this);
    }
    
    public ZonedDateTime atZoneSameInstant(ZoneId zoneId) {
        return ZonedDateTime.ofInstant(this.dateTime, this.offset, zoneId);
    }
    
    public ZonedDateTime atZoneSimilarLocal(ZoneId zoneId) {
        return ZonedDateTime.ofLocal(this.dateTime, zoneId, this.offset);
    }
    
    public OffsetTime toOffsetTime() {
        return OffsetTime.of(this.dateTime.toLocalTime(), this.offset);
    }
    
    public ZonedDateTime toZonedDateTime() {
        return ZonedDateTime.of(this.dateTime, this.offset);
    }
    
    public Instant toInstant() {
        return this.dateTime.toInstant(this.offset);
    }
    
    public long toEpochSecond() {
        return this.dateTime.toEpochSecond(this.offset);
    }
    
    @Override
    public int compareTo(OffsetDateTime offsetDateTime) {
        int n = compareInstant(this, offsetDateTime);
        if (n == 0) {
            n = this.toLocalDateTime().compareTo(offsetDateTime.toLocalDateTime());
        }
        return n;
    }
    
    public boolean isAfter(OffsetDateTime offsetDateTime) {
        long epochSecond = this.toEpochSecond();
        long epochSecond2 = offsetDateTime.toEpochSecond();
        return epochSecond > epochSecond2 || (epochSecond == epochSecond2 && this.toLocalTime().getNano() > offsetDateTime.toLocalTime().getNano());
    }
    
    public boolean isBefore(OffsetDateTime offsetDateTime) {
        long epochSecond = this.toEpochSecond();
        long epochSecond2 = offsetDateTime.toEpochSecond();
        return epochSecond < epochSecond2 || (epochSecond == epochSecond2 && this.toLocalTime().getNano() < offsetDateTime.toLocalTime().getNano());
    }
    
    public boolean isEqual(OffsetDateTime offsetDateTime) {
        return this.toEpochSecond() == offsetDateTime.toEpochSecond() && this.toLocalTime().getNano() == offsetDateTime.toLocalTime().getNano();
    }
    
    @Override
    public boolean equals(Object o) {
        if (this == o) {
            return true;
        }
        if ((o instanceof OffsetDateTime)) {
            OffsetDateTime offsetDateTime = (OffsetDateTime)o;
            return this.dateTime.equals(offsetDateTime.dateTime) && this.offset.equals(offsetDateTime.offset);
        }
        return false;
    }
    
    @Override
    public int hashCode() {
        return this.dateTime.hashCode() ^ this.offset.hashCode();
    }
    
    @Override
    public String toString() {
        return this.dateTime.toString() + this.offset.toString();
    }
    
    private Object writeReplace() {
        return new Ser((byte)10, this);
    }
    
    private void readObject(ObjectInputStream objectInputStream) throws InvalidObjectException {
        throw new InvalidObjectException("Deserialization via serialization delegate");
    }
    
    void writeExternal(ObjectOutput objectOutput) throws IOException {
        this.dateTime.writeExternal(objectOutput);
        this.offset.writeExternal(objectOutput);
    }
    
    static OffsetDateTime readExternal(ObjectInput objectInput) throws IOException, ClassNotFoundException {
        return of(LocalDateTime.readExternal(objectInput), ZoneOffset.readExternal(objectInput));
    }
    
    static {
        MIN = LocalDateTime.MIN.atOffset(ZoneOffset.MAX);
        MAX = LocalDateTime.MAX.atOffset(ZoneOffset.MIN);
    }
}
